﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ChartProblems
{
    public partial class Form1 : Form
    {
        private static Random rnd = new Random();

        private static System.Drawing.Font _chartFont = new System.Drawing.Font("Arial Narrow", 14);

        private static System.Drawing.Font _xaxis = new System.Drawing.Font("Arial Narrow", 14);

        private static object ChartLock = new object();

        private static int LoopCount = 1200;

        public Form1()
        {
            InitializeComponent();

            Telerik.WinControls.ThemeResolutionService.AllowAnimations = false;
        }

        private void ChartThread(object clearSeries)
        {
            var doClear = Convert.ToBoolean(clearSeries);

            for (int x = 0; x < LoopCount; x++)
            {
                lock (ChartLock)
                {
                    var chart = new Telerik.WinControls.UI.RadChartView();

                    try
                    {
                        GenerateChart(chart);
                    }
                    catch (Exception ex)
                    {
                        throw;
                    }
                    finally
                    {
                        try
                        {
                            if (doClear)
                            {
                                for (int y = chart.Series.Count - 1; y >= 0; y--)
                                {
                                    chart.Series[y].DataPoints.Clear();
                                }
                            }

                            chart.Dispose();
                        }
                        catch (Exception ex)
                        {
                            throw;
                        }
                    }
                }
            }
        }

        private static void GenerateChart(Telerik.WinControls.UI.RadChartView chart)
        {
            var seriesColors = new System.Drawing.Color[] { System.Drawing.Color.LightBlue, System.Drawing.Color.DarkRed, System.Drawing.Color.Black };
            var seriesNames = new string[] { "yield", "rate", "spread" };
            var seriesAbbrs = new string[] { "Yield", "Rate", "Spread" };

            List<ChartDataItem>[] seriesData = new List<ChartDataItem>[3];
            for (int x = 0; x < seriesData.Length; x++)
            {
                seriesData[x] = new List<ChartDataItem>();
            }

            for (int x = 0; x < 17; x++)
            {
                var date = DateTime.Now.AddDays(x);

                seriesData[0].Add(new ChartDataItem(date, rnd.NextDouble()));
                seriesData[1].Add(new ChartDataItem(date, rnd.NextDouble()));
                seriesData[2].Add(new ChartDataItem(date, rnd.NextDouble()));
            }

            chart.Width = 890;
            chart.Height = 320;

            // Set legend properties
            chart.ShowLegend = false;

            var xAxis = new Telerik.WinControls.UI.DateTimeCategoricalAxis();
            xAxis.AxisType = Telerik.Charting.AxisType.First;
            xAxis.Font = _xaxis;
            xAxis.LabelFormat = @"{0:MMM-yy}";
            xAxis.LabelFitMode = Telerik.Charting.AxisLabelFitMode.Rotate;
            xAxis.LastLabelVisibility = Telerik.Charting.AxisLastLabelVisibility.Visible;
            chart.Axes.Add(xAxis);

            var yAxis = new Telerik.WinControls.UI.LinearAxis();
            yAxis.AxisType = Telerik.Charting.AxisType.Second;
            yAxis.Font = _chartFont;
            yAxis.HorizontalLocation = Telerik.Charting.AxisHorizontalLocation.Left;
            yAxis.LabelFormat = "{0:###,###,##0.00}";
            chart.Axes.Add(yAxis);

            var myLine = new Telerik.WinControls.UI.LineSeries();
            myLine.Name = seriesNames[2];
            myLine.LegendTitle = seriesAbbrs[2];
            myLine.BackColor = seriesColors[2];
            myLine.BorderColor = seriesColors[2];
            myLine.CombineMode = Telerik.Charting.ChartSeriesCombineMode.None;
            myLine.DataSource = seriesData[2];
            myLine.ValueMember = @"Value";
            myLine.CategoryMember = @"ValueDate";
            myLine.HorizontalAxis = (Telerik.WinControls.UI.CategoricalAxis)chart.Axes[0];
            myLine.VerticalAxis = (Telerik.WinControls.UI.LinearAxis)chart.Axes[1];
            chart.Series.Add(myLine);

            for (int x = (seriesData.Length - 2); x >= 0; x--)
            {
                var mySeries = new Telerik.WinControls.UI.BarSeries();
                mySeries.Name = seriesNames[x];
                mySeries.LegendTitle = seriesAbbrs[x];
                mySeries.BackColor = seriesColors[x];
                mySeries.BorderColor = System.Drawing.Color.Black;
                mySeries.CombineMode = Telerik.Charting.ChartSeriesCombineMode.Stack;
                mySeries.DataSource = seriesData[x];
                mySeries.ValueMember = @"Value";
                mySeries.CategoryMember = @"ValueDate";
                mySeries.HorizontalAxis = (Telerik.WinControls.UI.CategoricalAxis)chart.Axes[0];
                mySeries.VerticalAxis = (Telerik.WinControls.UI.LinearAxis)chart.Axes[1];

                chart.Series.Add(mySeries);
            }

            DrawHorizontalSections(chart);

            chart.ChartElement.BackColor = System.Drawing.Color.LightGray;
            chart.View.Margin = new System.Windows.Forms.Padding(0);



            chart.AccessibleRole = System.Windows.Forms.AccessibleRole.None;
            chart.AllowDrop = false;
            chart.AllowShowFocusCues = false;
            chart.Annotations.Clear();
            chart.AutoScroll = false;
            chart.DisableGesture(Telerik.WinControls.GestureType.All);
            chart.EnableAnalytics = false;
            chart.EnableCodedUITests = false;
            chart.EnableTheming = false;
            chart.EnableRadAccessibilityObjects = false;
            chart.EnableKeyMap = false;
            chart.ShowDrillNavigation = false;
            chart.ShowPanZoom = false;
            chart.ShowToolTip = false;
            chart.ShowTrackBall = false;
            chart.UseWaitCursor = false;

            SaveChartToDisk(chart, @"C:\Temp\" + Guid.NewGuid().ToString() + @".png");
        }

        private static void DrawHorizontalSections(Telerik.WinControls.UI.RadChartView chart)
        {
            var area = chart.GetArea<Telerik.WinControls.UI.CartesianArea>();
            area.ShowGrid = true;

            var grid = area.GetGrid<Telerik.WinControls.UI.CartesianGrid>();
            grid.AlternatingHorizontalColor = true;
            grid.DrawHorizontalStripes = true;

            grid.ForeColor = System.Drawing.Color.Transparent;
            grid.BorderDashStyle = System.Drawing.Drawing2D.DashStyle.Custom;
            grid.BorderWidth = 0.0f;

            grid.AlternatingVerticalColor = false;
            grid.DrawVerticalFills = false;
            grid.DrawVerticalStripes = false;
        }

        private static void SaveChartToDisk(Telerik.WinControls.UI.RadChartView chart, string outputFile)
        {
            if (System.IO.File.Exists(outputFile))
            {
                System.IO.File.Delete(outputFile);
            }

            using (var bmp = new System.Drawing.Bitmap(chart.ClientSize.Width, chart.ClientSize.Height))
            {
                chart.DrawToBitmap(bmp, chart.Bounds);

                using (var stream = new System.IO.FileStream(outputFile, System.IO.FileMode.Create, System.IO.FileAccess.Write))
                {
                    bmp.Save(stream, System.Drawing.Imaging.ImageFormat.Png);
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            var threads = new System.Threading.Thread[3];
            int x;
            
            for (x = 0; x < threads.Length; x++)
            {
                threads[x] = new System.Threading.Thread(ChartThread);
                threads[x].Start(false);
            }

            for (x = 0; x < threads.Length; x++)
            {
                threads[x].Join();
            }

            MessageBox.Show("Done");
        }

        private void button2_Click(object sender, EventArgs e)
        {
            var threads = new System.Threading.Thread[3];
            int x;

            for (x = 0; x < threads.Length; x++)
            {
                threads[x] = new System.Threading.Thread(ChartThread);
                threads[x].Start(true);
            }

            for (x = 0; x < threads.Length; x++)
            {
                threads[x].Join();
            }

            MessageBox.Show("Done");
        }
    }

    public sealed class ChartDataItem
    {
        public DateTime ValueDate { get; set; }
        public double Value { get; set; }

        public ChartDataItem(DateTime valueDate, double value)
        {
            this.ValueDate = valueDate;
            this.Value = value;
        }
    }
}
